from math import acos,pi,asin
from d3g.distance import lineToLine,pointToLine,pointToPlane,p2p
from d3g.vector import magnitude,dotProduct,crossProduct
from d3g.angle import lines,planes,lineAndPlane



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Jun 20 02:08:56 2020

@author: Vaishakh
"""

def magnitude( vector = [1, 2, 3] ):
    """
    

    Parameters
    ----------
    vector : TYPE-> list
        DESCRIPTION-> list of direction ratios of a given vector. 
        The default is [1, 2, 3].

    Returns
    -------
    result : TYPE-> float
        DESCRIPTION-> Magnitude of a given vector.
        
    Example:
        Find the magnitude of the vector v = 5i + 3j - 4k
        
        Formula:
            √ ( A**2 + B**2 + C**2 )
        
        >>from d3g.vector import magnitude
        >>magnitude(vector = [ 5, 3, -4 ])
        7.0710678118654755
        
    """
    
    result=0
    for i in vector:
        result+=i**2
    result=result**0.5
    return result

def dotProduct(vector1 = [1, 2, 3], vector2 = [2, 3, 1]):
    """
    

    Parameters
    ----------
    vector1 : TYPE-> list
        DESCRIPTION-> list of direction ratios of a given vector. 
        The default is [1, 2, 3].
    vector2 : TYPE-> list
        DESCRIPTION-> list of direction ratios of a given vector. 
        The default is [2, 3, 1].

    Returns
    -------
    result : Datatype of the content of the list
        DESCRIPTION-> The dot product of two vectors.
        
    Example:
        Find the dot product of the vectors i + 2j + 3k and 2i + 3j + k
        
        >>from d3g.vector import dotProduct
        >>dotProduct(vector1 = [1, 2, 3], vector2 = [2, 3, 1])
        11

    """
    
    result=0
    for i,j in zip(vector1,vector2):
        result+=i*j
    return result

def crossProduct(vector1 = [1, 2, 3], vector2 = [2, 3, 1]):
    """
    

    Parameters
    ----------
    vector1 : TYPE-> list
        DESCRIPTION-> list of direction ratios of a given vector. 
        The default is [1, 2, 3].
    vector2 : TYPE-> list
        DESCRIPTION-> list of direction ratios of a given vector. 
        The default is [2, 3, 1].

    Returns
    -------
    result : list
        DESCRIPTION-> The list of direction ratios of the 
        cross product of two vectors.
        
    Example:
        Find the cross product of the vectors i + 2j + 3k and 2i + 3j + k
        
        >>from d3g.vector import crossProduct
        >>crossProduct(vector1 = [1, 2, 3], vector2 = [2, 3, 1])
        [-7, 5, -1]

    """
    a=vector1[0]
    b=vector1[1]
    c=vector1[2]
    d=vector2[0]
    e=vector2[1]
    f=vector2[2]
    result=[b*f-c*e,-(a*f-c*d),a*e-b*d]
    return result


    



from setuptools import setup
from os import path
this_directory = path.abspath(path.dirname("__file__"))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
  name = 'd3g',         
  packages = ['d3g'],   
  version = '0.3',      
  license='GPL 2.0',        
  description = 'Explore 3D geometry in python over here!',   
  author = 'Vaishakh Nargund',                  
  author_email = 'vaishakh.nargund1999@gmail.com', 
  long_description=long_description,
  long_description_content_type="text/markdown",     
  url = 'https://github.com/vaish1999/3D-Geometry-in-Python',  
  download_url = 'https://github.com/vaish1999/3D-Geometry-in-Python/archive/v2.2.tar.gz',   
  keywords = ['3d', 'geometry', 'line','plane','distance','python','vision'],   
  install_requires=[],
  classifiers=[
    'Development Status :: 3 - Alpha',     
    'Intended Audience :: Developers',     
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',   
    'Programming Language :: Python :: 3',      
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
  ],
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Jun 20 12:08:41 2020

@author: Vaishakh
"""

import math
from d3g.vector import magnitude,dotProduct

def lines(line1 = [[0,0,0],[1,1,1]],line2 = [[0,0,0],[2,2,2]]):
    """
    

    Parameters
    ----------
    line1 : TYPE-> list of 2 sublists
        DESCRIPTION-> The list of 2 sublists where first sublist is a 
        list of coordinates of the point line passing through and second 
        sublist is a list of direction ratios of the line.
        The default is [[0,0,0],[1,1,1]].
    line2 : TYPE-> list of 2 sublists
        DESCRIPTION-> The list of 2 sublists where first sublist is a 
        list of coordinates of the point line passing through and second 
        sublist is a list of direction ratios of the line.
        The default is [[0,0,0],[1,1,1]].

    Returns
    -------
    result : TYPE-> degree
        DESCRIPTION-> Angle between two lines.
        
    Example:
        Find the angle between the lines,
                x + 3 = y - 5 = z + 6
        line1: ------  ------  ------ 
                  2       4       2
        
                x - 5 = y + 4 = z + 7
        line2: ------  ------  ------ 
                  3       5       1
                  
        Formula:
           cos θ =  | b1 . b2 |
                   ______________
                    | b1 || b2 |
        
	>>from d3g.angle import lines
        >>lines(line1 = [ [-3, 5, -6], [2, 4, 2] ], line2 = [ [5, -4, -7], [3, 5, 1] ])
        14.963217433307127
        
    """
    numerator=abs(dotProduct(line1[1], line2[1]))
    denominator=magnitude(line1[1])*magnitude(line2[1])
    result=numerator/denominator
    result=180*math.acos(result)/math.pi
    return result

def planes(plane1 = [1, 1, 1, 1], plane2 = [1, 2, 3, 4]):
    """
    

    Parameters
    ----------
    plane1 : TYPE-> list
        DESCRIPTION -> list of 3 direction ratios of the normal of 
        the plane and the intercept. 
        The default is [1, 1, 1, 1].
    plane2 : TYPE-> list
        DESCRIPTION -> list of 3 direction ratios of the normal of 
        the plane and the intercept. 
        The default is [1, 2, 3, 4]..

    Returns
    -------
    result : TYPE-> degree
        DESCRIPTION-> Angle between two planes.
        
    Example:
        Find the angle between the planes,
        plane1: x + y + z = 1
        plane2: x + 2y + 3z = 4
                  
        Formula:
           cos θ =  | n1 . n2 |
                   ______________
                    | n1 || n2 |
                    
	>>from d3g.angle import planes
        >>planes(plane1 = [1, 1, 1, 1], plane2 = [1, 2, 3, 4])
        22.207654298596495

    """
    
    plane1=plane1[:-1]
    plane2=plane2[:-1]
    numerator=abs(dotProduct(plane1, plane2))
    denominator=magnitude(plane1)*magnitude(plane2)
    result=numerator/denominator
    result=180*math.acos(result)/math.pi
    return result

def lineAndPlane(line = [[0,0,0],[1,1,1]], plane = [1, 2, 3, 4]):
    """
    

    Parameters
    ----------
    line : TYPE-> list of 2 sublists
        DESCRIPTION-> The list of 2 sublists where first sublist is a 
        list of coordinates of the point line passing through and second 
        sublist is a list of direction ratios of the line.
        The default is [[0,0,0],[1,1,1]].
    plane : TYPE-> list
        DESCRIPTION -> list of 3 direction ratios of the normal of 
        the plane and the intercept. 
        The default is [1, 2, 3, 4]..

    Returns
    -------
    result : TYPE-> degree
        DESCRIPTION-> Angle between a line and a plane.
        
    Example:
        Find the angle between the planes,
                x + 3 = y - 5 = z + 6
        line: ------  ------  ------ 
                  2       4       2
                  
        plane: x + y + z = 1
                  
        Formula:
           sin θ =  | b . n |
                   ______________
                    | b || n |
                    
	>>from d3g.angle import lineAndPlane
        >>lineAndPlane(line = [[0,0,0],[1,1,1]], plane = [1, 2, 3, 4])
        70.52877936550934

    """
    
    plane=plane[:-1]
    numerator=abs(dotProduct(line[1], plane))
    denominator=magnitude(line[1])*magnitude(plane)
    result=numerator/denominator
    result=180*math.asin(result)/math.pi
    return result
    



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jun 19 01:08:48 2020

@author: Vaishakh
"""

from d3g.vector import magnitude,crossProduct,dotProduct

def _determinant(matrix = [ [1, 0, 0], [0, 1, 0], [0, 0, 1] ]):
    a=matrix[0][0]
    b=matrix[0][1]
    c=matrix[0][2]
    d=matrix[1][0]
    e=matrix[1][1]
    f=matrix[1][2]
    g=matrix[2][0]
    h=matrix[2][1]
    i=matrix[2][2]
    
    return (a*(e*i-f*h)-b*(d*i-f*g)+c*(d*h-e*g))
    

def p2p(p1=(1,1,1),p2=(0,0,0)):
    """
    

    Parameters
    ----------
    p1 :TYPE->tuple
        DESCRIPTION->tuple of coordinates of first point(x1,y1,z1) 
        The default is [1,1,1].
    p2 :TYPE->tuple
        DESCRIPTION->tuple of coordinates of second point(x2,y2,z2) 
        The default is [0,0,0].

    Returns
    -------
    result :TYPE->float
              DESCRIPTION->euclidean distance between the two given points p1 and p2
              
              
    Example: Find the distance between the points (2,1,0) and (5,1,4)
        
	>>from d3g.distance import p2p
        >>p2p(p1=(2,1,0),p2=(5,1,4))
        25.0
        

    """
    
    difference=[i-j for i,j in zip(p1,p2)]
    result=magnitude(difference)
    return result

def pointToPlane(p1=(0,0,0),plane=[1,1,1,1]):
    """
    

    Parameters
    ----------
    p1 : TYPE-> tuple
        DESCRIPTION-> tuple of coordinates of point (x1,y1,z1) 
                      from the respective given plane. 
        The default is (0,0,0).
    plane : TYPE-> list
        DESCRIPTION -> list of 3 direction ratios of the normal of 
        the plane and the intercept. 
        The default is [1,1,1,1].

    Returns
    -------
    result : TYPE-> float
        DESCRIPTION-> distance of the point p1 from the given plane.
        
    Example: Find the distance between the point (1,2,3) and the plane
             2x + 3y + 4z = 1
             
             Formula:
                 distance = | Ax1 + By1 + Cz1 - D |
                            _______________________
                            
                            √ ( A**2 + B**2 + C**2 )
                            
                    where, point->(x1,y1,z1) and plane equation is
                    Ax1 + By1 + Cz1 - D = 0
        
        >>from d3g.distance import pointToPlane
        >> pointToPlane(p1 = (5,2,3), plane = [2,3,4,1])
        0.9310344827586207

    """
    
    numerator,denominator=-plane[-1],0
    for i,j in zip(p1,plane):
        numerator+=i*j
    denominator=magnitude(plane[:-1])
    result=abs(numerator)/denominator
    return result

def pointToLine(p1=(0,0,0),line=[[0,0,0],[1,1,1]]):
    """
    

    Parameters
    ----------
    p1 : TYPE-> tuple
        DESCRIPTION-> tuple of coordinates of point (x1,y1,z1) 
                      from the respective given plane. 
        The default is (0,0,0).
    line : TYPE-> list of 2 sublists
        DESCRIPTION-> The list of 2 sublists where first sublist is a 
        list of coordinates of the point line passing through and second 
        sublist is a list of direction ratios of the line.
        The default is [[0,0,0],[1,1,1]].

    Returns
    -------
    result : TYPE-> float
        DESCRIPTION-> distance of the point p1 from the given line.
        
    Example: Find the distance between the point (1,2,3) and the line
             x + 3 = y - 5 = z + 6
            ------  ------  ------
              2       4       2
            
            Formula:
                
                k = | A(x1 - x2) + B(y1 - y2) + C(z1 - z2) |
                    ________________________________________
                            
                     √ ( A**2 + B**2 + C**2 )
                
                distance = p2p(p1,(A*k + x2, B*k + y2, C*k + z2))
                
        >>from d3g.distance import pointToLine
        >>pointToLine(p1 = (1, 2, 3), line = [ [-3, 5, -6], [2, 4, 2] ])
        14.899105176791087        

    """
    
    numerator,denominator=0,0
    for i,j,k in zip(line[1],p1,line[0]):
        numerator+=i*(j-k)
    denominator=magnitude(line[1])
    k=abs(numerator)/denominator
    pointOnPlane=[]
    for i,j in zip(line[1],line[0]):
        pointOnPlane.append((i*k)+j)
    pointOnPlane=tuple(pointOnPlane)
    result=p2p(p1,pointOnPlane)
    return result
    
def lineToLine(line1=[[0,0,0],[1,1,1]],line2=[[0,0,0],[2,2,2]]):
    """
    

    Parameters
    ----------
    line1 : TYPE-> list of 2 sublists
        DESCRIPTION-> The list of 2 sublists where first sublist is a 
        list of coordinates of the point line passing through and second 
        sublist is a list of direction ratios of the line.
        The default is [[0,0,0],[1,1,1]].
    line2 : TYPE-> list of 2 sublists
        DESCRIPTION-> The list of 2 sublists where first sublist is a 
        list of coordinates of the point line passing through and second 
        sublist is a list of direction ratios of the line.
        The default is [[0,0,0],[1,1,1]].

    Returns
    -------
    result : TYPE-> float
        DESCRIPTION-> distance between two lines.
        
    Example:
        Find the distance between the lines:
                x + 3 = y - 5 = z + 6
        line1: ------  ------  ------ 
                  2       4       2
        
                x - 5 = y + 4 = z + 7
        line2: ------  ------  ------ 
                  3       5       1
        
        Formula:
            parallel lines:
                d = | b x (a2 - a1) |
                     ______________
                         | b |
            skew lines:
                d = | (b1 x b2) . (a2 - a1) |
                     ______________________
                         | (b1 x b2) | 
        >>from d3g.distance import lineToLine
        >>lineToLine(line1 = [ [-3, 5, -6], [2, 4, 2] ], line2 = [ [5, -4, -7], [3, 5, 1] ])
        10.9577109184094

    """
    
    difference=[i-j for i,j in zip(line1[0],line2[0])]
    if line1[1]==line2[1]:                          #if lines are parallel
        numerator=crossProduct(difference, line1[1])
        numerator=magnitude(numerator)
        denominator=magnitude(line1[1])
        result=numerator/denominator
        return result
    directionRatioCrossProduct=crossProduct(line1[1], line2[1])
    numerator=abs(dotProduct(directionRatioCrossProduct, difference))             #if lines are skew
    denominator=magnitude(directionRatioCrossProduct)
    result=numerator/denominator
    return result
    
    
